/***
 * Gets the date as a string formatted to how the ReST service expects a date.
 * 
 * @param date  The date to format.
 * @returns {String}  The formatted date.
 */
function getDateString(date) {
    var res = '';
    res += (date.getMonth() + 1);
    res += '-' + date.getDate();
    res += '-' + date.getFullYear();
    res += '-' + date.getHours();
    res += '-' + date.getMinutes();
    res += '-' + date.getSeconds(); 
    res += '-' + getTimeZoneOffsetString(date.getTimezoneOffset());
    return res;
}

/**
 * Accepts the client's time zone offset from GMT in minutes as a parameter.
 * Returns the timezone offset in the format (+/-)DDDD 
*/
function getTimeZoneOffsetString(timezoneOffset)
{
    var hours = Math.floor(timezoneOffset / 60);
    var modMin = Math.abs(timezoneOffset % 60);
    var s = new String();
    s += (hours > 0) ? "-" : "%2B";
    var absHours = Math.abs(hours)
    s += (absHours < 10) ? "0" + absHours : absHours;
    s += ((modMin == 0) ? "00" : modMin);
    return(s);
}

/**
 * Gets the start date based on going back a number of units from today's date.
 * 
 * @param quantity  The number of units to go back.
 * @param units  The unit to go back by.  Should be one of {'minutes', 'hours', 'days'}.
 * @returns  The start date.
 */
function getStartDateFromToday(quantity, units) {
    var date = new Date();

    if (units == 'minutes') {
        date.setMinutes(date.getMinutes() - quantity);
    } else if (units == 'hours') {
        date.setHours(date.getHours() - quantity);
    } else if (units == 'days') {
        date.setDate(date.getDate() - quantity);
    }
    
    return date;
}

/***
 * Validates that the given dates are valid for a date filter.  Currently it just 
 * verifies that the start date occurs before the end date.
 * 
 * @param start  The start date
 * @param end  The end date
 * 
 * @returns {Boolean}  Whether the given date range is valid.
 */
function validateDateRange(start, end) {
    return start <= end;
}

/**
 * Pads a number by putting a zero in front of it if it is a one-digit number.
 * 
 * @param n The number to pad. Should be an non-negative integer.
 * @returns The padded number.
 */
function pad(n) {
    return ((""+n).length == 1 && n < 10) ? '0' + n : n;
}

//helper methods needed for the filter dialog

/**
 * Checks whether a given number is actually an integer.
 * 
 * @param n The number to check
 * @returns {Boolean} Whether the number is an integer
 */
function isInt(n) {     
    return parseFloat(n) == parseInt(n, 10) && !isNaN(n);
}

/**
 * Converts a display value for hours (1-12, AM/PM) into the DateTime value for hours (0-23)
 *  
 * @param hour  The display value for the hour of the day (1-12)
 * @param PM  Whether this is PM (false = AM, true = PM)
 * @returns  The value to use for a DateTime (0-23)
 */
function convertHours(hour, PM) {
    if (hour == 12)
        hour = 0;
    if (PM)
        hour += 12;

    return hour;
}

/**
 * Checks that the value currently assigned to the given input is acceptable 
 * for the minutes portion of a date (i.e. it is between 0 and 59).  In the 
 * event that it is not valid, it is set to a valid minute.
 * 
 * @param input The input whose value needs to be validated
 */
function validateMinutes(input) {
    if (!isInt(input.value) || input.value < 0) {
        input.value = 0;
    }
    else if (input.value > 59) {
        input.value = 59;
    }

    input.value = pad(input.value);
}

/**
 * Checks that the value currently assigned to the given input is acceptable 
 * for the hours portion of a date (i.e. it is between 1 and 12).  In the 
 * event that it is not valid, it is set to a valid hour.
 * 
 * @param input The input whose value needs to be validated
 */
function validateHours(input) {
    if (!isInt(input.value) || input.value < 1) {
        input.value = 1;
    }
    else if (input.value > 12) {
        input.value = 12;
    }

    input.value = pad(input.value);
}

/**
 * Gets a formatted string for a date.  The format should be MM/DD/YYYY hh:mm AM/PM.
 * 
 * @param date  The date to be formatted.
 * @returns {String}  The formatted string representation of the date.
 */
function getDateFormatted(date) {
    var res = '';
    res += pad(date.getMonth() + 1);
    res += '/' + pad(date.getDate());
    res += '/' + date.getFullYear();

    var hours = date.getHours();
    if (hours < 12) {
        if (hours == 0) res += " 12";
        else res += " " + pad(hours);
        res += ":" + pad(date.getMinutes()) + " AM";
    }
    else {
        hours -= 12;
        if (hours == 0) res += " 12";
        else res += " " + pad(hours);
        res += ":" + pad(date.getMinutes()) + " PM";
    }

    return res;
}

/**
 * Takes a span whose innerHTML is the long integer representation of a date, and converts 
 * the innerHTML to be of the form '(MM/DD/YY hh:mm:ss AM/PM)'.
 * 
 * @param element
 */
function convertDateTime(element) {
    //if this has already been converted, it will have the leading parenthesis
    if (element.innerHTML[0] == '(')
        return;
    
    element.innerHTML = '(' + formatLongToDate(element.innerHTML) + ')';
}

/**
 * Gets whether the custom range has been chosen in the date range dialog.
 * 
 * @returns Whether the custom range has been chosen in the date range dialog.
 */
function isDialogCustomRange() {
    return hqDojo.byId('rbRange').checked;
}

/**
 * Sets whether the dialog radio buttons should be set to a custom range or not.
 * 
 * @param isCustomRange  Whether the dialog radio buttons should be set to a custom range or not.
 */
function setDialogCustomRange(isCustomRange) {
    //jabosk: this is a hack.  the two date text boxes don't set their date until after 
    //the dialog is done showing, and it is triggering the method regardless of what I do.
    //So we have to ignore the first two times this method is called.
    if (++setCustomRangeCalls > 2) {
        hqDojo.byId('rbRange').checked = isCustomRange;
        hqDojo.byId('rbCurrent').checked = !isCustomRange;
    }
}

var setCustomRangeCalls = 0;

/**
 * Gets the value chosen for the quantity in the date range selector.
 * Should be one of {4, 8, 12, 24, 30, 48, 60, 90, 120}.
 * 
 * @returns  The quantity.
 */
function getDialogQuantity() {
    var dlgQuantitySelect = hqDojo.byId('advDialogQuantitySelect');
    
    return dlgQuantitySelect.options[dlgQuantitySelect.selectedIndex].value;
}

/**
 * Gets the value chosen for the timespan in the date range selector.
 * Should be one of {'Minute', 'Hour', 'Day'}.
 * 
 * @returns  The timespan.
 */
function getDialogTimespan() {
    var dlgTimespanSelect = hqDojo.byId('advDialogTimespanSelect');
    
    return dlgTimespanSelect.options[dlgTimespanSelect.selectedIndex].value;
}

/**
 * Gets the start date chosen in the date range dialog.
 * 
 * @returns The start date.
 */
function getDialogStartDate() {
    var startDate = hqDijit.byId('advDialogDateStart').get("value");
    startDate.setMinutes(hqDojo.byId('advDialogDateStartMinute').value);
    
    var startHour = parseInt(hqDojo.byId('advDialogDateStartHour').value);
    var startAMPM = hqDojo.byId('advDialogDateStartAMPM');
    startDate.setHours(convertHours(startHour, startAMPM.options[startAMPM.selectedIndex].value == "PM"));
    
    return startDate;
}

/**
 * Gets the end date chosen in the date range dialog.
 * 
 * @returns The end date.
 */
function getDialogEndDate() {
    var endDate = hqDijit.byId('advDialogDateEnd').get("value");            
    endDate.setMinutes(hqDojo.byId('advDialogDateEndMinute').value);

    var endHour = parseInt(hqDojo.byId('advDialogDateEndHour').value);
    var endAMPM = hqDojo.byId('advDialogDateEndAMPM');
    endDate.setHours(convertHours(endHour, endAMPM.options[endAMPM.selectedIndex].value == "PM"));
    
    return endDate;
}

/**
 * Sets the custom range components of the date range dialog to 
 * reflect the give start date and end date.
 * 
 * @param startDate The start date of the range.
 * @param endDate The end date of the range.
 */
function setDialogDateRange(startDate, endDate) {
    hqDijit.byId('advDialogDateStart').attr('value', startDate);
    hqDijit.byId('advDialogDateEnd').attr('value', endDate);
    hqDojo.byId('advDialogDateStartMinute').value = pad(startDate.getMinutes());
    hqDojo.byId('advDialogDateEndMinute').value = pad(endDate.getMinutes());

    if (startDate.getHours() < 12) {
        hqDojo.byId('advDialogDateStartHour').value = startDate.getHours() == 0 ? 12 : pad(startDate.getHours());
        hqDojo.byId('advDialogDateStartAMPM').value = "AM";
    }
    else {
        hqDojo.byId('advDialogDateStartHour').value = startDate.getHours() == 12 ? 12 : pad(startDate.getHours() - 12);
        hqDojo.byId('advDialogDateStartAMPM').value = "PM";
    }          

    if (endDate.getHours() < 12) {
        hqDojo.byId('advDialogDateEndHour').value = endDate.getHours() == 0 ? 12 : pad(endDate.getHours());
        hqDojo.byId('advDialogDateEndAMPM').value = "AM";
    }
    else {
        hqDojo.byId('advDialogDateEndHour').value = endDate.getHours() == 12 ? 12 : pad(endDate.getHours() - 12);
        hqDojo.byId('advDialogDateEndAMPM').value = "PM";
    }
}

/**
 * Sets the dropdown components of the date range dialog to reflect the given quantity and timespan options. 
 * 
 * @param quantity The quantity to be selected.  Should be one of {4, 8, 12, 24, 30, 48, 60, 90, 120}.
 * @param timespan The timespan to be selected.  Should be one of {'Minutes', 'Hours', 'Days'}
 */
function setDialogDropdowns(quantity, timespan) {
    dlgQuantitySelect = hqDojo.byId('advDialogQuantitySelect');
    dlgTimespanSelect = hqDojo.byId('advDialogTimespanSelect');

    dlgQuantitySelect.value = quantity;
    dlgTimespanSelect.value = timespan;
}